require "import"
import "android.media.MediaRecorder"
import "java.io.File"
import "android.widget.GridView"
import "android.widget.ArrayAdapter"
import "com.androlua.LuaDialog"

local folderPath = "/storage/emulated/0/Music/"
local defaultFormat = "mp3"
local defaultBitrate = 128000
local defaultSampleRate = 44100

_G.selectedFormat = _G.selectedFormat or defaultFormat
_G.selectedBitrate = _G.selectedBitrate or defaultBitrate
_G.selectedSampleRate = _G.selectedSampleRate or defaultSampleRate

local function getFilePath()
    local fileName = "Recording_" .. os.date("%Y-%m-%d_%H-%M-%S") .. "." .. _G.selectedFormat
    return folderPath .. fileName
end

local function tableIndexOf(tbl, value)
    for i, v in ipairs(tbl) do
        if v == value then return i end
    end
    return nil
end

local folder = File(folderPath)
if not folder.exists() then
    folder.mkdirs()
end

if _G.recorder == nil then
    _G.recorder = MediaRecorder()
    _G.isRecording = false
    _G.isPaused = false
end

function startRecording()
    if _G.isRecording then
        service.speak("Recording is already in progress.")
        return
    end

    local filePath = getFilePath()

    _G.recorder.setAudioSource(MediaRecorder.AudioSource.MIC)
    _G.recorder.setOutputFormat(MediaRecorder.OutputFormat.MPEG_4)
    _G.recorder.setAudioEncoder(MediaRecorder.AudioEncoder.AAC)

    -- بٹ ریٹ اور سیمپلنگ ریٹ کو صحیح طور پر لاگو کرنا
    _G.recorder.setAudioEncodingBitRate(_G.selectedBitrate)
    _G.recorder.setAudioSamplingRate(_G.selectedSampleRate)
    _G.recorder.setOutputFile(filePath)

    pcall(function()
        _G.recorder.prepare()
        _G.recorder.start()
        _G.isRecording = true
        _G.isPaused = false
        service.speak("Recording started. Format: " .. _G.selectedFormat .. ", Bitrate: " .. (_G.selectedBitrate / 1000) .. "kbps, Sample Rate: " .. _G.selectedSampleRate .. "Hz")
    end)
end

function pauseRecording()
    if not _G.isRecording or _G.isPaused then
        service.speak("No recording is in progress or already paused.")
        return
    end

    pcall(function()
        _G.recorder.pause()
        _G.isPaused = true
        service.speak("Recording paused.")
    end)
end

function resumeRecording()
    if not _G.isRecording or not _G.isPaused then
        service.speak("Recording is not paused.")
        return
    end

    pcall(function()
        _G.recorder.resume()
        _G.isPaused = false
        service.speak("Recording resumed.")
    end)
end

function stopRecording()
    if not _G.isRecording then
        service.speak("No recording is in progress.")
        return
    end

    pcall(function()
        _G.recorder.stop()
        _G.recorder.release()
        _G.recorder = MediaRecorder()
        _G.isRecording = false
        _G.isPaused = false
        service.speak("Recording stopped and saved.")
    end)
end

function showSelectionDialog(title, options, currentSelection, onSelect)
    local dlg = LuaDialog(service)
    dlg.setTitle(title .. " (Current: " .. currentSelection .. ")")
    dlg.setItems(options)
    dlg.setButton("Cancel", function() showMainMenu() end)

    dlg.setOnItemClickListener(function(parent, view, position, id)
        local choice = options[position + 1]
        if choice then
            onSelect(choice)
            service.speak(title .. " set to " .. tostring(choice))
            showMainMenu()
        else
            service.speak("Invalid selection.")
        end
    end)

    dlg.show()
end

function selectFormat()
    local formats = {"mp3", "m4a", "3gp", "wav"}
    showSelectionDialog("Select Format", formats, _G.selectedFormat, function(choice)
        _G.selectedFormat = choice
    end)
end

function selectBitrate()
    local bitrates = {32000, 64000, 96000, 128000, 160000, 192000, 224000, 256000, 320000}
    local bitrateLabels = {"32kbps", "64kbps", "96kbps", "128kbps", "160kbps", "192kbps", "224kbps", "256kbps", "320kbps"}

    showSelectionDialog("Select Bitrate", bitrateLabels, (_G.selectedBitrate / 1000) .. "kbps", function(choice)
        local index = tableIndexOf(bitrateLabels, choice)
        if index then
            _G.selectedBitrate = bitrates[index]
        end
    end)
end

function selectSampleRate()
    local sampleRates = {8000, 16000, 22050, 32000, 44100, 48000}
    local sampleRateLabels = {"8000Hz", "16000Hz", "22050Hz", "32000Hz", "44100Hz", "48000Hz"}

    showSelectionDialog("Select Sample Rate", sampleRateLabels, _G.selectedSampleRate .. "Hz", function(choice)
        local index = tableIndexOf(sampleRateLabels, choice)
        if index then
            _G.selectedSampleRate = sampleRates[index]
        end
    end)
end

function showMainMenu()
    local buttons = {
        "Start Recording",
        "Pause Recording",
        "Resume Recording",
        "Stop Recording",
        "Select Format (Current: " .. _G.selectedFormat .. ")",
        "Select Bitrate (Current: " .. (_G.selectedBitrate / 1000) .. "kbps)",
        "Select Sample Rate (Current: " .. _G.selectedSampleRate .. "Hz)"
    }

    local layout = {
        GridView,
        id = "grid",
        numColumns = 1,
        layout_width = "fill",
        layout_height = "fill"
    }

    local dlg = LuaDialog(service)
    dlg.setTitle("Voice Recorder")
    dlg.setView(loadlayout(layout))
    dlg.setButton("Close", nil)
    dlg.show()

    grid.adapter = ArrayAdapter(service, android.R.layout.simple_list_item_1, String(buttons))

    grid.onItemClick = function(parent, view, position, id)
        dlg.dismiss()
        task(500, function()
            if position == 0 then startRecording()
            elseif position == 1 then pauseRecording()
            elseif position == 2 then resumeRecording()
            elseif position == 3 then stopRecording()
            elseif position == 4 then selectFormat()
            elseif position == 5 then selectBitrate()
            elseif position == 6 then selectSampleRate()
            else service.speak("Unknown option.") end
        end)
    end
end

showMainMenu()